@@ -1,5 +1,5 @@
 ##***********************************************************************
-## $Id: utils.R 67 2011-03-14 14:20:45Z mariotomo $
+## $Id: utils.R 92 2011-08-09 14:00:44Z mariotomo $
 ##
 ## this file is part of the R library delftfews.  delftfews is free
 ## software: you can redistribute it and/or modify it under the terms
@@ -101,19 +101,10 @@ stretches <- function(input, gap=1, what="start", zero.surrounded=FALSE) {
   return(result)
 }
 
-rollapply.delftfews <- function(data, ..., na.pad=TRUE, align='right') {
-  class.data <- class(data)
-  index.data <- index(data)
-  result <- NextMethod(na.pad=na.pad, align=align)
-  class(result) <- class.data
-  index(result) <- index.data
-  return(result)
-}
-
 rollingSum <- function(data, width, na.action=na.zero) {
   ## commodity function
   ## na.zero specifies that NA will be summed as zero.
-  rollapply(na.action(data), width, FUN=sum)
+  rollapply(na.action(data), width, FUN=sum, fill=NA, align='right')
 }
 
 shift.vector <- function(v, by) {
@@ -151,8 +142,6 @@ get.step <- function(L, require.constant=FALSE)
   UseMethod('get.step')
 
 get.step.default <- function(L, require.constant=FALSE) {
-  ## not exported, tested.
-
   ## returns the value of the most common difference between
   ## subsequent elements.
 
@@ -182,29 +171,6 @@ sum.first <- function(input, count=12) {
   colSums(input[1:count,])
 }
 
-rollapply.default <- function(data, width, FUN, ...) {
-  ## returns the rolling application of `FUN` to data (nth element in
-  ## returned vector is `FUN` of width elements in data from n-width
-  ## to n.)
-
-  apply.na.action <- function(data, na.action=na.pass, ...) na.action(data)
-  data <- apply.na.action(data, ...)
-
-  ## width must be positive.
-  ## result is same length as data (starts with `width-1` NA).
-
-  len <- length(data)
-  if(width < 1) {
-    ## Only positive width allowed
-    return(rep(NA, len))
-  }
-  if(width > len) {
-    rep(NA, len)
-  } else {
-    c( rep(NA, width - 1) , apply(embed(data, width), 1, FUN) )
-  }
-}
-
 double.threshold <- function(data, threshold.false, threshold.true, initial.status=FALSE, on.equality=FALSE)
   UseMethod('double.threshold')
 
@@ -335,6 +301,6 @@ read.sheet <- function(file, sheet=NULL, header=TRUE, sep="\t",
              strip.white=strip.white, stringsAsFactors=stringsAsFactors, ...)
 }
 
-diff.delftfews <- function(x, ...) {
+diff.zoo <- function(x, ...) {
   diff(coredata(x))
 }
