@@ -1,5 +1,5 @@
 ##***********************************************************************
-## $Id: runit.utils.R 50 2010-10-26 07:51:26Z mariotomo $
+## $Id: runit.utils.R 94 2011-08-09 14:56:17Z mariotomo $
 ##
 ## this file is part of the R library delftfews.  delftfews is free
 ## software: you can redistribute it and/or modify it under the terms
@@ -192,37 +192,9 @@ test.rollingSum.delftfews <- function() {
   checkEquals(length(target.numeric), length(result))
   checkEqualsNumeric(target.numeric, result)
   target <- timeseries(from=0, by=720*60, length.out=20, b=target.numeric)
-  checkEquals(target, result)
-}
-
-test.rollapply <- function() {
-  input <- c(rep(1:3, 2), NA, 4, 4, 4)
-  ##  [1]  1  2  3  1  2  3 NA  4  4  4
-
-  result <- rollapply(input, 2, sum)
-  expect <- c(NA, 3, 5, 4, 3, 5, NA, NA, 8, 8)
-  checkEquals(expect, result)
-
-  result <- rollapply(input, 2, sum, na.action=na.zero)
-  expect <- c(NA, 3, 5, 4, 3, 5, 3, 4, 8, 8)
-  checkEquals(expect, result)
-
-  result <- rollapply(input, 4, sum, na.action=na.zero)
-  expect <- c(NA, NA, NA, 7, 8, 9, 6, 9, 11, 12)
-  checkEquals(expect, result)
-  
-  result <- rollapply(input, 4, max, na.action=na.zero)
-  expect <- c(NA, NA, NA, 3, 3, 3, 3, 4, 4, 4)
-  checkEquals(expect, result)
-  
-  result <- rollapply(input, 4, mean, na.action=na.zero)
-  expect <- c(NA, NA, NA, 1.75, 2.00, 2.25, 1.50, 2.25, 2.75, 3.00)
-  checkEquals(expect, result)
-  
-  result <- rollapply(input, 2, min, na.action=na.zero)
-  expect <- c(NA, 1, 2, 1, 1, 2, 0, 0, 4, 4)
-  checkEquals(expect, result)
-  
+  checkEqualsNumeric(target, result)
+  checkEqualsNumeric(index(target), index(result))
+  ## checkEquals(target, result)
 }
 
 test.shift.vector <- function() {
